// Copyright 2003 Leigh L. Klotz, Jr.

package com.hipme.flipper;

import danger.app.Application;
import danger.app.AppResources;
import danger.app.DataStore;
import danger.app.Listener;
import danger.app.Registrar;
import danger.app.SettingsDB;
import danger.app.IPCMessage;
import danger.app.Event;
import danger.app.EventType;
import danger.app.SettingsDB;
import danger.app.SettingsDBException;
import danger.audio.Meta;
import danger.ui.RingTonePicker;
import danger.audio.RingToneObject;
import danger.audio.ToneClass;
import danger.internal.Date;
import danger.net.HiptopConnection;
import danger.net.HiptopTransaction;
import danger.ui.Bitmap;
import danger.ui.CheckBox;
import danger.ui.DateTimeEditor;
import danger.ui.DialogWindow;
import danger.ui.MarqueeAlert;
import danger.ui.NotificationManager;
import danger.ui.NumberEditor;
import danger.ui.Rect;
import danger.ui.Font;
import danger.ui.View;
import danger.ui.Pen;
import danger.ui.StaticText;
import danger.ui.Style;
import danger.ui.TextField;
import danger.util.DEBUG;
import danger.util.StringTokenizer;
import java.util.Vector;

/**
 * The Appointment record
 */
class Appointment extends Message implements Resources, Constants, Events {
  String title;
  String location;
  int startTime;
  int durationHours;
  int durationMinutes;
  int durationSeconds;
  String dateString;
  String timeString;
  Date date;
  Date time;

  Appointment() {
    super();
  }

  Appointment(String userName) {
    super(userName);
  }

  // Called in response to networkEvent HiptopTransaction WHAT_NEW_APPOINTMENT.
  // Possibly notifies user, and pops up a dialog box.
  void process(StringTokenizer st) {
    DEBUG.p("Appointment process  " + userName +" " + st);
    flipper app = (flipper)Application.getCurrentApp();
    title = st.nextToken();
    location = st.nextToken();
    startTime = Integer.valueOf(st.nextToken()).intValue();
    durationSeconds = Integer.valueOf(st.nextToken()).intValue();
    dateString = st.nextToken();
    timeString = st.nextToken();
    date = new Date(startTime);
    time = date;
    durationHours = durationSeconds / 3600;
    durationMinutes = (durationSeconds - (durationHours * 3600))/60;
    debugPrint("ht st" + this);
    app.setWho(userName);
    String notice = userName + " has flipped you: " + title + " on " + dateString+" @ " + timeString;
    app.notifyAlert(userName, notice);
    DialogWindow receiveNewAppointmentDialog = app.getDialog(kReceiveNewAppointmentDialog, this);
    ((TextField)(receiveNewAppointmentDialog.getDescendantWithID(kReceiveNewAppointmentTitle))).setText(title);
    if (location != null)
      ((TextField)(receiveNewAppointmentDialog.getDescendantWithID(kReceiveNewAppointmentLocation))).setText(location);
    ((DateTimeEditor)(receiveNewAppointmentDialog.getDescendantWithID(kReceiveNewAppointmentDate))).setDate(date);
    ((DateTimeEditor)(receiveNewAppointmentDialog.getDescendantWithID(kReceiveNewAppointmentTime))).setDate(time);
    ((NumberEditor)(receiveNewAppointmentDialog.getDescendantWithID(kReceiveNewAppointmentDurationHours))).setValue(durationHours);
    ((NumberEditor)(receiveNewAppointmentDialog.getDescendantWithID(kReceiveNewAppointmentDurationMinutes))).setValue(durationMinutes);
    ((StaticText)(receiveNewAppointmentDialog.getDescendantWithID(kReceiveNewAppointmentUserName))).setText(userName);
    receiveNewAppointmentDialog.show();
  }

  Appointment parseReceiveDialog(DialogWindow receiveNewAppointmentDialog) {
    title = receiveNewAppointmentDialog.getDescendantWithID(kReceiveNewAppointmentTitle).toString();
    location = receiveNewAppointmentDialog.getDescendantWithID(kReceiveNewAppointmentLocation).toString();
    date = ((DateTimeEditor)(receiveNewAppointmentDialog.getDescendantWithID(kReceiveNewAppointmentDate))).getDate();
    time = ((DateTimeEditor)(receiveNewAppointmentDialog.getDescendantWithID(kReceiveNewAppointmentTime))).getDate();
    startTime = date.getUnixTimeGMT(); // what about the time?  how does it know?
    userName = receiveNewAppointmentDialog.getDescendantWithID(kReceiveNewAppointmentUserName).toString();
    durationHours = ((NumberEditor)(receiveNewAppointmentDialog.getDescendantWithID(kReceiveNewAppointmentDurationHours))).getValue();
    durationMinutes = ((NumberEditor)(receiveNewAppointmentDialog.getDescendantWithID(kReceiveNewAppointmentDurationMinutes))).getValue();
    durationSeconds = durationHours*60*60 + durationMinutes*60;
    dateString = date.getDateString();
    timeString = time.getTimeString();
    debugPrint("parseReceiveDialog " + this);
    return this;
  }

  Appointment parseSendDialog(DialogWindow sendNewAppointmentDialog) {
    title =  sendNewAppointmentDialog.getDescendantWithID(kSendNewAppointmentTitle).toString();
    location =  sendNewAppointmentDialog.getDescendantWithID(kSendNewAppointmentLocation).toString();
    date = ((DateTimeEditor)(sendNewAppointmentDialog.getDescendantWithID(kSendNewAppointmentDate))).getDate();
    time = ((DateTimeEditor)(sendNewAppointmentDialog.getDescendantWithID(kSendNewAppointmentTime))).getDate();
    startTime = date.getUnixTimeGMT(); // what about the time?  how does it know?
    userName = sendNewAppointmentDialog.getDescendantWithID(kSendNewAppointmentUserName).toString();
    durationHours =  ((NumberEditor)(sendNewAppointmentDialog.getDescendantWithID(kSendNewAppointmentDurationHours))).getValue();
    durationMinutes =  ((NumberEditor)(sendNewAppointmentDialog.getDescendantWithID(kSendNewAppointmentDurationMinutes))).getValue();
    durationSeconds = durationHours*60*60 + durationMinutes*60;
    debugPrint("parseSendDialog " + this);
    return this;
  }

  public String toString() {
    return "#<" + super.toString() + " " +
      ("startTime="+startTime) + " " +
      ("datestring = " + dateString) + " " +
      ("durationHours="+durationHours) + " " +
      ("durationMinutes="+durationMinutes) + " " +
      ("durationSeconds="+durationSeconds) + " " +
      ("location = " + location) + " " +
      ("startTime = " + startTime) + " " +
      ("time = " + time) + " " +
      ("timestring = " + timeString) + " " +
      ("title="+title) + " " +
      ("userName = " + userName) + ">";
  }

  void debugPrint(String m) {
    // DEBUG.p(toString());
  }

  public boolean receiveEvent(Event event) {
    boolean consumed = false;
    switch (event.type) {
    case kShowSendNewAppointmentEvent:
      consumed = true;
      showSendNewAppointment(event);
      break;
    case kHandleSendNewAppointmentEvent:
      consumed = true;
      handleSendNewAppointment(event);
      break;
    case kHandleReceiveNewAppointmentEvent:
      consumed = true;
      handleReceiveNewAppointment(event);
      break;
    case kHandleReceiveNewAppointmentDecline:
      consumed = true;
      handleDeclineNewAppointment(event);
      break;
    default:
      consumed = super.receiveEvent(event);
    }
    return consumed;
  }

  public void showSendNewAppointment(Event event) {
    flipper app = (flipper)Application.getCurrentApp();
    DialogWindow sendNewAppointmentDialog = app.getDialog(kSendNewAppointmentDialog, this);
    Date now = new Date();
    now.setMinutes(0);
    now.addHours(1);
    ((DateTimeEditor)(sendNewAppointmentDialog.getDescendantWithID(kSendNewAppointmentDate))).setDate(now);
    ((DateTimeEditor)(sendNewAppointmentDialog.getDescendantWithID(kSendNewAppointmentTime))).setDate(now);
    if (app.getWho() != null)
      ((TextField)sendNewAppointmentDialog.getDescendantWithID(kSendNewAppointmentUserName)).setText(app.getWho());
    sendNewAppointmentDialog.show();
  }

  void handleSendNewAppointment(Event event) {
    DialogWindow sendNewAppointmentDialog = (DialogWindow)(event.argument);
    flipper app = (flipper)Application.getCurrentApp();
    parseSendDialog(sendNewAppointmentDialog);
    app.setWho(userName);
    // what|title|location|startTimeInt|durationSeconds|dateString|timeString
    // TODO: handle '|' in title and location
    // Also, StringTokenizer can't handle successive delimiters.
    String t = ("".equals(title)) ? "(none)" : title;
    String l = ("".equals(location)) ? " " : location;
    if (! flipOut(WHAT_NEW_APPOINTMENT, 
		t + "|" + 
		l + "|" +
		startTime + "|" + 
		durationSeconds + "|" + 
		date.getDateString() + "|" + 
		time.getTimeString())) {
      Meta.play(Meta.BEEP_ACTION_FAIL);
      return;
    }
    CheckBox meToo = (CheckBox)(sendNewAppointmentDialog.getDescendantWithID(kSendNewAppointmentMeTooCheckbox));
    if (meToo.getValue() != 0) {
      addNewAppointment(false);
      app.flipWindow.flipIn(userName, "Added to your your calendar as well.");
    }
  }

  void addNewAppointment(boolean from) {
    IPCMessage ipcmessage = new IPCMessage();
    ipcmessage.addItem("action" , "new");
    ipcmessage.addItem("title" , title);
    ipcmessage.addItem("start" , startTime);
    // ipcmessage.addItem("end" , null);
    ipcmessage.addItem("duration" , durationSeconds);
    // ipcmessage.addItem("allday" , null);
    ipcmessage.addItem("location" , location);
    if (from) 
      ipcmessage.addItem("notes" , "Flipped from " + userName); // I18N      
    else
      ipcmessage.addItem("notes" , "Flipped to " + userName); // I18N
    Registrar.sendMessage("Calendar" , ipcmessage , null);
  }

  /**
   * Called in response to kHandleReceiveNewAppointmentEvent from 
   * confirmation dialog kReceiveNewAppointmentDialog, and actually
   * adds the appointment, confirms back, and removes the item from the queue.
   */
  void handleReceiveNewAppointment(Event event) {
    DialogWindow receiveNewAppointmentDialog = ((DialogWindow)(event.argument));
    parseReceiveDialog(receiveNewAppointmentDialog);
    flipper app = (flipper)Application.getCurrentApp();
    app.setWho(userName);
    String notice =
      "Appointment: " + title +
      " on " + dateString +
      " @" + timeString;
    DEBUG.p("handleReceiveNewAppointment: " + notice);
    app.flipWindow.flipIn(userName, notice);
    addNewAppointment(false);
    app.flipWindow.flipOut(WHAT_MESSAGE, "Accepted " + notice);
    // Flush the queued request and process the next one, if any.
    app.pendingQueue.dequeue();
    app.flipCheck();
  }

  void handleDeclineNewAppointment(Event event) {
    DialogWindow receiveNewAppointmentDialog = ((DialogWindow)(event.argument));
    flipper app = (flipper)Application.getCurrentApp();
    parseReceiveDialog(receiveNewAppointmentDialog);
    app.setWho(userName);
    app.flipWindow.flipOut(WHAT_MESSAGE, "Declined " + getNotice());
    app.pendingQueue.dequeue();
    app.flipCheck();
  }

  String getNotice() {
    return
      "Appointment: " + title +
      " on " + dateString +
      " @" + timeString;
  }



}
