// Copyright 2003 Leigh L. Klotz, Jr.

package com.hipme.flipper;

import danger.app.*;
import danger.internal.Date;
import danger.ui.*;
import danger.system.*;
import danger.audio.Meta;
import java.util.Vector;
import danger.util.DEBUG;

class FlipWindow extends ScreenWindow implements Resources, Events {
  final static int INPUT_LINE_HEIGHT = 20;

  Rect bounds;
  Font font;
  HistoryView historyView;
  InputLine inputLine;
  String who = null;

  // How do I get these into the resource file?
  final static int kCmd_ReturnToLauncher = 600;

  public FlipWindow() {
    bounds = getBounds();
    font = Font.findBoldSystemFont();
    buildActionMenu();
    buildHistoryView();
    buildInputLine();
    showAllChildren();
    setFocusedChild(inputLine);
  }

  final void buildActionMenu() {
    Menu menu = getActionMenu();

    menu.removeAllItems();
    menu.addFromResource(Application.getCurrentApp().getResources(), kMainMenuResID);
  }

  final void buildHistoryView() {
    int width = (bounds.right - bounds.left);
    int height = (bounds.bottom - bounds.top);
    historyView = new HistoryView();
    historyView.setPosition(0,0);
    historyView.setSize(width, height-INPUT_LINE_HEIGHT);
    addChild(historyView);
    historyView.show();
  }

  final void buildInputLine() {
    int width = (bounds.right - bounds.left);
    int height = (bounds.bottom - bounds.top);
    inputLine = new InputLine(this);
    inputLine.setSize(width, INPUT_LINE_HEIGHT);
    inputLine.setPosition(0, height - INPUT_LINE_HEIGHT);
    addChild(inputLine);
  }

  public boolean receiveEvent(Event e) {
    boolean consumed = false;
    switch (e.type) {
    default:
      consumed = super.receiveEvent(e);
    }
    return consumed;
  }


  public boolean eventWidgetUp(int inWhichWidget, Event inEvent) {
    boolean consumed = false;

    switch (inWhichWidget)
      {
      case Event.DEVICE_WHEEL:
	break;
      case Event.DEVICE_MULTIPLE_WHEEL:
	break;
      case Event.DEVICE_WHEEL_BUTTON:
	break;
      case Event.DEVICE_ARROW_UP:
	break;
      case Event.DEVICE_ARROW_DOWN:
	break;
      case Event.DEVICE_ARROW_LEFT:
	break;
      case Event.DEVICE_ARROW_RIGHT:
	break;
      }
    return (consumed || super.eventWidgetUp(inWhichWidget, inEvent));
  }

  String getWho() {
    return who;
  }

  void setWho(String who) {
    this.who=who;
    setSubTitle(who);
  }

  public boolean flipOut(byte what, String message) {
    DEBUG.p(this + " flipOut " + message);
    String who = getWho();
    if (who == null) {
      Meta.play(Meta.BEEP_ACTION_FAIL);
      return false;
    }
    historyView.myAddItem(who + "<" + message);
    ((flipper)Application.getCurrentApp()).flipOut(what, message);
    setFocusedChild(inputLine);
    return true;
  }

  public void flipIn(String username, String message) {
    DEBUG.p(this + " flipIn " + message);
    setWho(username);
    historyView.myAddItem(username + ">" + message);
    setFocusedChild(inputLine);
  }

  public boolean eventWidgetDown(int inWhichWidget, Event inEvent) {
    boolean	consumed = false;
    switch (inWhichWidget)
      {
      case Event.DEVICE_WHEEL:
	break;
      case Event.DEVICE_MULTIPLE_WHEEL:
	break;
      case Event.DEVICE_WHEEL_BUTTON:
	break;
      case Event.DEVICE_ARROW_UP:
	break;
      case Event.DEVICE_ARROW_DOWN:
	break;
      case Event.DEVICE_ARROW_LEFT:
	break;
      case Event.DEVICE_ARROW_RIGHT:
	break;
      case Event.DEVICE_BUTTON_BACK:
	new Event(Application.getCurrentApp(), 
		  kCmd_ReturnToLauncher).sendToTarget();
	consumed = true;
	break;
      }
    return (consumed || super.eventWidgetDown(inWhichWidget, inEvent));
  }
}
