// Copyright 2003 Leigh L. Klotz, Jr.

package com.hipme.flipper;

import danger.app.Application;
import danger.app.AppResources;
import danger.app.DataStore;
import danger.app.Listener;
import danger.app.Registrar;
import danger.app.SettingsDB;
import danger.app.IPCMessage;
import danger.app.Event;
import danger.app.EventType;
import danger.app.SettingsDB;
import danger.app.SettingsDBException;
import danger.audio.Meta;
import danger.ui.AlertWindow;
import danger.util.format.StringFormat;
import danger.util.StringUtils;
import danger.ui.RingTonePicker;
import danger.audio.RingToneObject;
import danger.audio.ToneClass;
import danger.internal.Date;
import danger.net.HiptopConnection;
import danger.net.HiptopTransaction;
import danger.ui.Bitmap;
import danger.ui.CheckBox;
import danger.ui.DialogWindow;
import danger.ui.MarqueeAlert;
import danger.ui.NotificationManager;
import danger.ui.NumberEditor;
import danger.ui.Rect;
import danger.ui.Font;
import danger.ui.View;
import danger.ui.Pen;
import danger.ui.StaticText;
import danger.ui.StaticTextBox;
import danger.ui.Style;
import danger.ui.TextField;
import danger.util.DEBUG;
import danger.util.StringTokenizer;
import java.util.Vector;

/**
 * The Application class.
 *
 */
public class flipper extends Application implements Resources, Constants, Events
{
  static FlipWindow flipWindow;	// only one for now
  MarqueeAlert marqueeAlert = null;
  boolean pendingIconVisible = false;
  boolean inForeground = false;
  PendingQueue pendingQueue = null;

  final static String RECEIVED_FLIP_RINGTONE_ID = "receivedFlipRingToneID";
  final static String RECEIVED_FLIP_RINGTONE_FLAGS = "receivedFlipRingToneFlags";
  SettingsDB settings;
  final static String SETTINGS_DB="flipper";
  RingToneObject receivedFlipRingTone = null;
  Bitmap splashScreenBackground;
  
  /**
   * Creates the main application class.
   */
  public flipper() {
    super();
    checkSettings();
    Friends.checkFriends();
    pendingQueue = new PendingQueue(this);
    updatePreviewScreen();
    setReceiveConnectivityEvents(true);
    try {
      NotificationManager.registerPendingIcon("flipper", 
					      getBitmap(kIDFlipperPendingNotificationIcon),
					      getBitmap(kIDFlipperPendingNotificationIconInChooser));
    } catch (Exception e) { }
  }

  private void checkSettings() {
    boolean oldSettings = SettingsDB.findDB(SETTINGS_DB);
    settings = new SettingsDB(SETTINGS_DB, true);
    if (oldSettings) {
      restoreSettings();
    } else {
      initSettings();
    }
  }

  private void initSettings() {
    restoreSettings();
  }

  private void restoreSettings() {
    int id = Meta.INVALID;
    int flags = 0;
    try { id = settings.getIntValue(RECEIVED_FLIP_RINGTONE_ID); }
    catch (SettingsDBException settingsdbexception) { }
    try { flags = settings.getIntValue(RECEIVED_FLIP_RINGTONE_FLAGS); }
    catch (SettingsDBException settingsdbexception) { }
    receivedFlipRingTone = new RingToneObject();
    receivedFlipRingTone.setFilterFlags(flags);
    receivedFlipRingTone.setID(id);
  }

  public FlipWindow getMainWindow() {
    if (flipWindow == null) {
      flipWindow = new FlipWindow();
      flipWindow.setTitle(Application.getCurrentApp().getString(kStrWindowTitle));
    }
    return flipWindow;
  }

  /**
   * Handles the Suspend event.
   *
   */
  public void suspend() {
    inForeground = false;
    super.suspend();
  }

  public final void renderSplashScreen(View view, Pen pen) {
    try {
      DEBUG.p("flipper: renderSplashScreen");
      // First draw the background
      if (splashScreenBackground == null)
	splashScreenBackground = getBitmap(AppResources.ID_SPLASH_SCREEN);
      pen.drawBitmap(0, 0, splashScreenBackground);
      int n = pendingQueue == null ? 0 : pendingQueue.getSize();
      Rect rect = view.getBounds();
      Font font = Font.findBoldOutlineSystemFont();
      int leading = font.getAscent()+font.getDescent()+1;
      int x = rect.left + 6;
      int y = rect.top + 30;
      // I18N
      pen.setColor(view.getStyle().getColor(Style.SPLASH_SCREEN_TEXT_COLOR));
      pen.setTextOutlineColor(view.getStyle().getColor(Style.SPLASH_SCREEN_TEXT_OUTLINE_COLOR));
      pen.setFont(font);
      pen.setTextOutlineColor(view.getStyle().getColor(Style.SPLASH_SCREEN_TEXT_OUTLINE_COLOR));
      pen.drawText(x, y, getString(AppResources.ID_APP_NAME));
      y += leading;

      if (n == 0) 
	return;

      String s = n + " unread:";
      if (s != null) {
	pen.drawText(x, y, s);
	y += leading;
      }
      x += 3;
      pen.drawText(x, y, peekSummary());
      y += leading;
      if (n > 1) {
	pen.drawText(x, y, "...");
      }
    }
    catch (Exception e) {
      DEBUG.p("Exception in renderSplashScreen " + e);
      e.printStackTrace();
    }
  }


  public boolean receiveEvent(Event event) {
    boolean consumed = false;
    switch (event.type) {
    case EventType.EVENT_CONNECTION_UP:
      DEBUG.p("flipper: connection up");
      break;
    case EventType.EVENT_DATASTORE_RESTORED:
      consumed = super.receiveEvent(event);
      flipCheck();
      updatePreviewScreen();
      DEBUG.p("flipper: datastore " + event.argument + " restored");
      break;
    case kAboutEvent:
      showAbout();
      consumed = true;
      break;
    case FlipWindow.kCmd_ReturnToLauncher:
      consumed = true;
      returnToLauncher();
      break;
    case kShowSettingsEvent:
      showSettings();
      consumed = true;
      break;
    case kSettingsDoneEvent:
      handleSettings(event);
      consumed = true;
      break;
    case kShowAddNewFriendEvent:
      showAddNewFriend();
      consumed = true;
      break;
    case kAddNewFriendDoneEvent:
      handleAddNewFriend(event);
      consumed = true;
      break;
    case kShowEditFriendEvent:
      showEditFriend();
      consumed = true;
      break;
    case kEditFriendDoneEvent:
      handleEditFriend(event);
      consumed = true;
      break;
    case kNewSessionEvent:
      consumed = true;
      showNewSession(event);
      break;
    case kSetUserNameEvent:
      consumed = true;
      handleSetUserName(event);
      break;
    case kShowSendNewAppointmentEvent:
    case kHandleSendNewAppointmentEvent:
    case kHandleReceiveNewAppointmentEvent:
    case kHandleReceiveNewAppointmentDecline:
      consumed = true;
      new Appointment().receiveEvent(event);
      break;
    case kShowFlipPhotoEvent:
    case kHandleFlipPhotoEvent:
      consumed = new Photo(getWho()).receiveEvent(event);
      break;
    }
    if (! consumed)
      consumed = super.receiveEvent(event);
    return consumed;
  }

  public void showNewSession(Event event) {
    DialogWindow newSessionDialog = getDialog(kNewSessionDialog, this);
    newSessionDialog.show();
  }

  private void handleSetUserName(Event event) {
    DialogWindow newSessionDialog = ((DialogWindow)event.argument);
    setWho(newSessionDialog.getDescendantWithID(kNewSessionUserName).toString());
  }

  boolean verifyFromWhom(String fromWhom, byte[] record) {
    int namelen = ((int)record[1]);
    int namestart = 2;
    String embeddedUsername = new String(record, namestart, namelen);
    return (fromWhom.equals(embeddedUsername));
  }

  public final void networkEvent(Object item) {
    if (item instanceof HiptopTransaction) {
      HiptopTransaction ht = ((HiptopTransaction)item);
      String fromWhom = ht.getFromUsername();
      DEBUG.p("networkEvent ht.getFromUsername="+fromWhom);
      Friend friend = Friends.getFriend(fromWhom);
      if (verifyFromWhom(fromWhom, ht.getBytes())) {
	if (friend != null && ! friend.isBlocked()) {
	  pendingQueue.enqueue(ht.getBytes());
	  updatePreviewScreen();
	} else {
	  DEBUG.p("Dropped blocked message " + fromWhom);
	}
      } else {
	DEBUG.p("Dropped forged message from " + fromWhom);
      }
      flipCheck();
    } else {
      DEBUG.p("passing up " + item);
      super.networkEvent(item);
    }
  }

  String peekSummary() {
    byte[] record = pendingQueue.peekBytes();
    if (record == null) return null;
    byte what = record[0];
    int namelen = ((int)record[1]);
    int namestart = 2;
    String fromUsername = new String(record, namestart, namelen);
    String type = null;
    switch (what) {
    case WHAT_MESSAGE:
      type = "message";	// I18N
      break;
    case WHAT_NEW_APPOINTMENT: 
      type = "appointment";	// I18N
      break;
    case WHAT_NEW_PHOTO: 
      type = "photo";		// I18N
      break;
    }
    if (type == null) return fromUsername;
    return type + " from " + fromUsername; // I18N
  }

  final void flipCheck() {
    DEBUG.p("flipCheck");
    boolean done = false;
    while (!done) {
      byte[] record = pendingQueue.peekBytes();
      if (record == null) {
	DEBUG.p("flipCheck bytes = null");
	// Cancel the alert if we are in the foreground and find no more stuff.
	if (inForeground) {
	  cancelAlert();
	  setPendingIconVisible(false);
	} 
	break;
      }
      DEBUG.p("flipCheck bytes[0] = " + record[0]+" length = " + record.length);
      // leave pending icon visible even in app if there are unprocessed 
      // entries in queue.  Eventually provide a menu item to trigger
      // processing intrusive ones (dialogs) rather than processing
      // them always.
      setPendingIconVisible(true);
      // bytes: 
      // byte 0=type
      // byte 1 = name length
      // byte 2 = first byte of name
      // byte 2+namelen = last byte of name
      // byte 2+namelen+1 ... end = image bytes
      byte what = record[0];
      int namelen = ((int)record[1]);
      int namestart = 2;
      String fromUsername = new String(record, namestart, namelen);
      int start = namestart+namelen;
      // the username field was verified by networkEvent and is trustworthy.
      switch (what) {
      case WHAT_MESSAGE:
	{
	  new Message(fromUsername).handleReceiveNewMessage(new String(record, start, record.length-start));
	  done = false;
	  break;
	}
      case WHAT_NEW_APPOINTMENT: 
	{
	  StringTokenizer st = new StringTokenizer(new String(record, start, record.length-start), "|", false);
	  new Appointment(fromUsername).process(st);
	  done = true;
	  break;
	}
      case WHAT_NEW_PHOTO: 
	{
	  new Photo(fromUsername).handleReceiveNewPhoto(record, start);
	  done = true;
	  break;
	}
      default:
	DEBUG.p("Discarding unknown message " + new String(record));
	pendingQueue.dequeue();
      }
    }
  }

  /**
   * Handles the Resume event.
   *
   */
  public void resume() {
    // Print the version for PDM testing.
    System.err.println("flipper: Version = " + getString(ID_VERSION));
    inForeground = true;
    // Create/get the main window...
    getMainWindow();
		
    // Show the main window...
    // mWindow.Update();
    flipWindow.show();

    // Process queued requests if any.
    flipCheck();
  }
	
  private void showAbout() {
    showAlert(kAboutAlert);
  }

  private void showSettings() {
    DialogWindow dw = getDialog(kSettingsDialog, this);
    RingTonePicker r = (RingTonePicker)(dw.getDescendantWithID(kSettingsRingTonePicker));
    r.setGroupFilter(ToneClass.BACKGROUND_DEFAULT);
    r.setRingTone(receivedFlipRingTone);
    dw.show();
  }

  private void handleSettings(Event e) {
    DialogWindow dw = (DialogWindow)(e.argument);
    RingTonePicker r = (RingTonePicker)(dw.getDescendantWithID(kSettingsRingTonePicker));
    receivedFlipRingTone = r.getRingTone();
    settings.setIntValue(RECEIVED_FLIP_RINGTONE_ID, receivedFlipRingTone.getID());
    settings.setIntValue(RECEIVED_FLIP_RINGTONE_FLAGS, receivedFlipRingTone.getFilterFlags());
  }

  private void showAddNewFriend() {
    showAddNewFriend(null);
  }

  private void showAddNewFriend(String name) {
    DialogWindow dw = getDialog(kAddNewFriendDialog, this);
    RingTonePicker r = (RingTonePicker)(dw.getDescendantWithID(kAddNewFriendDialogFlipRingTonePicker));
    r.setGroupFilter(ToneClass.BACKGROUND_DEFAULT);
    r.setRingTone(receivedFlipRingTone);
    if (name != null) {
      TextField tf = (TextField)(dw.getDescendantWithID(kAddNewFriendDialogHiptopName));
      tf.setText(name);
    }      
    dw.show();
  }

  private void handleAddNewFriend(Event e) {
    DialogWindow dw = (DialogWindow)(e.argument);
    RingTonePicker r = (RingTonePicker)(dw.getDescendantWithID(kAddNewFriendDialogFlipRingTonePicker));
    TextField tf = (TextField)(dw.getDescendantWithID(kAddNewFriendDialogHiptopName));
    String name = tf.toString();
    if (name == null || name.length()==0) {
      Meta.play(Meta.BEEP_ACTION_FAIL);
      return;
    }
    {
      Friend oldFriend = Friends.getFriend(name);
      if (oldFriend != null) {
	AlertWindow aw = getAlert(kOldFriendAlert, this);
	aw.setMessage(StringFormat.fromRsrc(kOldFriendAlertText, oldFriend.getName()));
	aw.show();
	return;
      }
    }
    Friends.addFriend(new Friend(name, r.getRingTone(), false));
  }

  private void showEditFriend() {
    String name = getWho();
    if (name == null) {
      Meta.play(Meta.BEEP_ACTION_FAIL);
      return;
    }
    Friend friend = Friends.getFriend(name);
    if (friend == null) {
      showAddNewFriend(name);
      return;
    }
    DialogWindow dw = getDialog(kEditFriendDialog, this);
    {
      RingTonePicker r = (RingTonePicker)(dw.getDescendantWithID(kEditFriendDialogFlipRingTonePicker));
      r.setGroupFilter(ToneClass.BACKGROUND_DEFAULT);
      RingToneObject rto = friend.getTone();
      if (rto != null)
	r.setRingTone(rto);
    }
    {
      CheckBox cb = (CheckBox)(dw.getDescendantWithID(kEditFriendDialogBlocked));
      cb.setValue(friend.isBlocked() ? 1 : 0);
    }
    {
      StaticText st = (StaticText)(dw.getDescendantWithID(kEditFriendDialogHiptopName));
      st.setText(friend.getName());
    }
    dw.show();
  }

  private void handleEditFriend(Event e) {
    DialogWindow dw = (DialogWindow)(e.argument);
    StaticText st = (StaticText)(dw.getDescendantWithID(kEditFriendDialogHiptopName));
    Friend friend = Friends.getFriend(st.getText());
    RingTonePicker r = (RingTonePicker)(dw.getDescendantWithID(kEditFriendDialogFlipRingTonePicker));
    friend.setTone(r.getRingTone());
    CheckBox cb = (CheckBox)(dw.getDescendantWithID(kEditFriendDialogBlocked));
    friend.setIsBlocked(cb.getValue() != 0);
    DEBUG.p("edited friend = " + friend);
    Friends.updateFriend(friend);
  }

  final void notifyAlert(String fromUsername, String notice) {
    setPendingIconVisible(true);
    if (notice.length() > 50) notice = notice.substring(0, 50) + "...";
    if (inForeground) {
      DEBUG.p("skipping marqueeAlert '" + notice+"'");
    } else {
      if (marqueeAlert == null) {
	marqueeAlert = new MarqueeAlert(notice, 1);
	DEBUG.p("Made new " + marqueeAlert + " notice="+notice);
      } else {
	marqueeAlert.setText(notice);
	marqueeAlert.setLoop(1);
	DEBUG.p("Reused " +marqueeAlert + " notice="+ notice);
      }
    }
    Friend friend = Friends.getFriend(fromUsername);
    if (friend == null) {
      Meta.play(receivedFlipRingTone);
    } else {
      DEBUG.p("Playing ringtone for " + friend);
      Meta.play(friend.getTone());
    }
    NotificationManager.marqueeAlertNotify(marqueeAlert);    
  }

  final void cancelAlert() {
    if (marqueeAlert != null)
      NotificationManager.marqueeAlertRemove(marqueeAlert);
  }

  final void setPendingIconVisible(boolean visible) {
    if (visible == pendingIconVisible) return;
    pendingIconVisible = visible;
    NotificationManager.setPendingIconVisible("flipper",
					      pendingIconVisible,
					      false);
  }


  public boolean flipOut(byte what, String message) {
    String to = getWho();
    if (to == null) return false;
    return new Message(to).flipOut(what, message);
  }

  void setWho(String who) {
    flipWindow.setWho(who);
  }

  String getWho() {
    if (flipWindow == null) return null;
    return flipWindow.getWho();
  }
}



/*
 * The queue of pending events
 */
class PendingQueue {
  final static String QUEUE_DATASTORE_NAME = "QUEUE";

  int getSize() {
    DataStore ds = getQueueDataStore();
    if (ds == null) return 0;
    return ds.getRecordCount();
  }

  PendingQueue(Listener listener) {
    DataStore ds = getQueueDataStore();
    if (ds == null) {
      DEBUG.p("flipper: No datastore found, must be either first time or we don't have it yet and how can we tell?");
      ds = DataStore.createDataStore(QUEUE_DATASTORE_NAME, true);
      DEBUG.p("flipper: ds = " +ds);
    } else {
      DEBUG.p("flipper: found ds = " + ds + " count = " + ds.getRecordCount());
    }
    ds.setAutoSyncNotifyee(listener);
    DataStore.registerListenerForAutoSyncDone(listener);
  }

  final DataStore getQueueDataStore() {
    DataStore ds = DataStore.findDataStore(QUEUE_DATASTORE_NAME);
    DEBUG.p("flipper: getQueueDataStore " + ds + 
	    ((ds == null) ? "" : " count = " + ds.getRecordCount()));
    return ds;
  }


  final void enqueue(byte[] record) {
    DataStore dsq = getQueueDataStore();
    int iq = dsq.addRecord(record);
    DEBUG.p("enq " + record + " ds.size=" + dsq.getRecordCount() + " record number = " + iq);
  }

  final byte[] peekBytes() {
    // since we always operate on record 0
    // it is safe to peek and dequeue later.
    // as record 0 won't be renumbered.
    DataStore ds = getQueueDataStore();
    if (ds == null) return null;
    byte []record = null;;
    int count = ds.getRecordCount();
    if (count == 0) return null;
    record = ds.getRecordData(0);
    DEBUG.p("peek record bytes=" + record.length);
    return record;
  }    

  final void dequeue() {
    DEBUG.p("dequeue");
    {
      DataStore dsq = getQueueDataStore();
      int count = dsq.getRecordCount();
      if (count > 0) 
	dsq.removeRecord(0);
      DEBUG.p("deq size=" + dsq.getRecordCount());
    }
  }
}


