// Copyright (c) 2003, 2004 Leigh L. Klotz, Jr. <klotz@graflex.org>
// 
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

package com.hipme.flipper;

import danger.audio.RingToneObject;
import danger.util.StringUtils;
import danger.util.StringTokenizer;
import danger.app.DataStore;
import java.util.Hashtable;
import danger.util.DEBUG;

class Friends {
  final static String FRIENDS_DATASTORE_NAME="friends";

  static Hashtable friends = new Hashtable();

  static void checkFriends() {
    DataStore ds = getFriendsDataStore();
    if (ds != null) {
      Friends.restoreFriends(ds.getRecords());
    } else {
      ds = DataStore.createDataStore(FRIENDS_DATASTORE_NAME, true);
    }
    restoreFriends(ds.getRecords());
  }

  static DataStore getFriendsDataStore() {
    return DataStore.findDataStore(FRIENDS_DATASTORE_NAME);
  }


  // How do we restore records that arrive later?
  static void restoreFriends(byte [][]records) {
    int count = records.length;
    for (int i = 0; i < count; i++) {
      Friend friend = new Friend(records[i]);
      friends.put(friend.name, friend);
    }
  }

  // probably want a sorted array instead
  static Hashtable getFriends() {
    return friends;
  }

  static Friend getFriend(String name) {
    return (Friend)(friends.get(name));
  }

  static void addFriend(Friend friend) {
    friends.put(friend.getName(), friend);
    DataStore ds = getFriendsDataStore();
    int recNo = ds.addRecord(friend.getBytes());
    int uid = ds.getRecordUID(recNo);
    friend.setUID(uid);
    DEBUG.p("Added uid="+uid+" as " + friend);
  }

  static void updateFriend(Friend friend) {
    friends.put(friend.getName(), friend);
    DataStore ds = getFriendsDataStore();
    int uid = friend.getUID();
    ds.setRecordData(uid, friend.getBytes());
    DEBUG.p("Updated uid="+uid+" to " + friend);
  }
}
