// Copyright (c) 2003, 2004 Leigh L. Klotz, Jr. <klotz@graflex.org>
// 
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

package com.hipme.flipper;

import danger.app.*;
import danger.internal.Date;
import danger.ui.*;
import danger.system.*;
import java.util.Vector;
import danger.util.DEBUG;

class InputLine extends TextField implements Events, Resources, Constants {
  FlipWindow flipWindow;

  InputLine(FlipWindow flipWindow) {
    this.flipWindow = flipWindow;
		
    setAllowCursorBlink(true);
    setAutoText(true);
    setAutoCap(true);
  }
	
  public void setText(String value)
  {
    super.setText(value);
    invalidate();
  }
	
  public boolean eventKeyUp(char c, Event event) {
    boolean consumed = super.eventKeyUp(c, event);
    if (c == 13) consumed = flipOut();
    return consumed;
  }
	
  public boolean eventWidgetUp(int widget, Event event) {
    boolean consumed = false;

    switch (widget) {
    case Event.DEVICE_WHEEL_BUTTON:
      super.eventWidgetUp(widget, event);
      consumed = flipOut();
      break;
    }
    return (consumed || super.eventWidgetUp(widget, event));
  }


  boolean flipOut() {
    boolean consumed = false;
    String value = getStringCRLF();
    if (!value.equals("")) 
      consumed = flipWindow.flipOut(WHAT_MESSAGE, value);
    setText(null);
    return consumed;
  }
}
